/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Christoph Renner
   co-programmer:
*/

#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_WORLD_ENTITY

#include "loading/factory.h"
#include "debug.h"
#include "loading/load_param.h"
#include "util/loading/load_param_xml.h"
#include "state.h"
#include "player.h"
#include "playable.h"

#include "actionbox_enemy.h"

ObjectListDefinition(ActionboxEnemy);
CREATE_FACTORY(ActionboxEnemy);

ActionboxEnemy::ActionboxEnemy(const TiXmlElement* root)
{
  PRINTF(0)("ActionboxEnemy\n");
  
  this->registerObject(this, ActionboxEnemy::_objectList);
  this->toList(OM_GROUP_00);
  
  this->isActive = true;
  
  //TODO init to zero for non-debug
  this->pitch = 0.0f;
  this->dPitch = PI;
  
  if ( root )
    this->loadParams( root );
}

ActionboxEnemy::~ActionboxEnemy()
{
}

void ActionboxEnemy::loadParams(const TiXmlElement* root)
{
  WorldEntity::loadParams( root );
}

void ActionboxEnemy::tick( float dt )
{
  this->pitch += this->dPitch*dt;
  
  moveTowardsBox( NULL, dt );
  return;
  
  if ( isActive && State::getActionBox() )
  {
    ActionBox* box = State::getActionBox();
    if ( box->isPointInBox( this->getAbsCoor() ) )
    {
      attackPlayer( box, dt );
    }
    else
    {
      moveTowardsBox( box, dt );
    }
  }
}

void ActionboxEnemy::attackPlayer( ActionBox * box, float dt )
{
}

void ActionboxEnemy::moveTowardsBox( ActionBox * box, float dt )
{
  Vector targetPos = State::getPlayer()->getPlayable()->getAbsCoor();
  
  Vector fw = this->getAbsDir().apply( Vector( 0, 0, -1 ) );
  fw.normalize();
  
  Vector dv = targetPos - this->getAbsCoor();
  dv.normalize();
  
  Vector up = fw.cross( dv )*-1;
  
  Quaternion rot(up, 10.0*dt);
  rot.normalize();
  Quaternion cur = this->getAbsDir();
  Quaternion ne = rot * cur;
  this->setAbsDir( ne );
  
  // 1, 0, 0
  // 0, 1, 0
  return;
  Vector k = ne.apply( Vector( 1, 0, 0 ) );
  k.normalize();
  
  Vector vr = k.cross( Vector( 0, 1, 0) );
  
  rot = Quaternion(vr, 10.0*dt);
  rot.normalize();
  cur = ne;
  ne = rot * cur;
  ne.normalize();
 
  this->setAbsDir( ne );
}

void ActionboxEnemy::draw( ) const
{
  Vector fw = this->getAbsDir().apply( Vector( 1, 0, 0 ) );
  fw.normalize();
  fw = fw * 100;
  
  Vector mp = this->getAbsCoor();
  Vector op = mp + fw;
  
  Vector targetPos = State::getPlayer()->getPlayable()->getAbsCoor();
  Vector dv = targetPos - this->getAbsCoor();
  dv.normalize();
  dv *= 100;
  dv += mp;
  
  Vector up = fw.cross( dv )*-1;
  up += mp;
  
  //PRINTF(0)("DEBUG\n");
  //mp.debug();
  //op.debug();
  
  glMatrixMode(GL_MODELVIEW);
  glPushMatrix();

  glPushAttrib(GL_ENABLE_BIT);

  glDisable(GL_LIGHTING);
  glDisable(GL_TEXTURE_2D);
  glDisable(GL_BLEND);
  glLineWidth(2.0);
  glColor3f(1.0, 0.0, 0.0 );
  
  
  glBegin(GL_LINE_STRIP);
    glVertex3f(mp.x, mp.y, mp.z);
    glVertex3f(op.x, op.y, op.z);
  glEnd();
  
  glBegin(GL_LINE_STRIP);
    glVertex3f(mp.x, mp.y, mp.z);
    glVertex3f(dv.x, dv.y, dv.z);
  glEnd();
  
  glBegin(GL_LINE_STRIP);
    glVertex3f(mp.x, mp.y, mp.z);
    glVertex3f(up.x, up.y, up.z);
  glEnd();

  glPopMatrix();
  
  WorldEntity::draw();
}
